<?php
/**
 * Plugin Name:         RS Word Count
 * Description:         Count words in posts and display them in a sortable column on the list of posts.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-word-count/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-word-count
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             0.5
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action( 'plugins_loaded', function() {
    class WordCountPlugin extends PluginBase {
        protected $plugin_file = __FILE__;

        use PluginBaseSettings;

        protected $default_post_types = [ 'post', 'page' ];
        protected $post_types;

        function get_post_types() {
            if ( empty( $this->post_types ) ) {
                $post_types = $this->get_option( 'post_types', $this->default_post_types );
                $this->post_types = apply_filters( 'word_count_post_types', $post_types );
            }
            return $this->post_types;
        }

        function post_type_ok( $post_type ) {
            return in_array( $post_type, $this->get_post_types() );
        }

        function calculate_word_count( $post ) {
            $post = get_post( $post );

            $wordcount = str_word_count( strip_tags( $post->post_content ) );
            update_post_meta( $post->ID, '_rswc_wordcount', $wordcount );

            return $wordcount;
        }


        /************************************************************************
         *
         *	Update word counts on changes
         *
         ************************************************************************/

        function do_transition_post_status_action_count_words( $new, $old, $post ) {
            if ( $this->post_type_ok( $post->post_type ) )
                $this->calculate_word_count( $post );
        }


        /************************************************************************
         *
         *	Word-count column in admin posts list
         *
         ************************************************************************/

        // Add the column

        function do_manage_posts_columns_filter( $columns, $post_type ) {
            if ( $this->post_type_ok( $post_type ) ) {
                $columns[ 'word_count' ] = __( 'Words', 'rs-word-count' );

                // Make sure everything is updated
                do {
                    $posts = get_posts( [
                        'post_type' => $post_type,
                        'post_status' => 'any',
                        'meta_key' => '_rswc_wordcount',
                        'meta_compare' => 'NOT EXISTS',
                    ] );

                    foreach ( $posts as $post ) {
                        $this->calculate_word_count( $post );
                    }
                } while ( !empty( $posts) );
            }
            return $columns;
        }

        function do_manage_pages_columns_filter( $columns ) {
            return $this->do_manage_posts_columns_filter( $columns, 'page' );
        }

        // Display the column content

        function do_manage_posts_custom_column_action( $column, $post_id) {
            if ( $column == 'word_count' ) {
                $post = get_post( $post_id );

                $wordcount = $post->_rswc_wordcount;
                if ( ! $wordcount  )
                    $wordcount = $this->calculate_word_count( $post );

                echo number_format_i18n( $wordcount );
            }
        }

        function do_manage_pages_custom_column_action( $column, $post_id) {
            $this->do_manage_posts_custom_column_action( $column, $post_id );
        }


        // Make the column narrower
        function do_admin_head_action_post_table_css() {
            echo '<style type="text/css">';
            echo ".wp-list-table .column-word_count { width: 10% !important; overflow: hidden; }";
            echo '</style>';
        }



        // Make column sortable

        protected $init_wordcount_column_sortable_callback_priority = 20;

        function do_init_action_wordcount_column_sortable() {
            foreach ( $this->get_post_types() as $post_type ) {
                add_filter( "manage_edit-{$post_type}_sortable_columns", function ( $columns ) {
                    $columns['word_count'] = 'word_count';
                    return $columns;
                } );
            }
        }

        function do_pre_get_posts_action_wordcount_sorting( $query ) {
            if ( ! is_admin() ) return;

            $orderby = $query->get( 'orderby' );
            if ( $orderby == 'word_count' ) {
                $query->set( 'meta_key', '_rswc_wordcount' );
                $query->set( 'orderby', 'meta_value_num' );
            }
        }

        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'Word count', 'rs-word-count' ),
                __( 'RS Word count', 'rs-word-count' )
            );
        }

        function settings_define_sections_and_fields() {

            // Post types

            $section = $this->settings_add_section(
                'section_post_types',
                __( 'Post-types', 'rs-word-count' ),
                __( 'Select the post-types you want to count words in', 'rs-word-count' )
            );

            $this->settings_build_post_types_menu(
                'post_types', $section,
                __( 'Post-types', 'rs-word-count' ),
                [ 'public' => true ],
                $this->default_post_types,
                [
                    'rs_word_count_post_types',
                    'rs_custom_post_types',
                ]
            );
        }
    }

    new WordCountPlugin();
} );
